MOSAMATIC2_BUNDLE_IDENTIFIER = 'org.mumc'
MOSAMATIC2_APP_NAME = 'mosamatic2'
MOSAMATIC2_VERSION_FILE = 'mosamatic2/ui/resources/VERSION'
MOSAMATIC2_IMAGES_DIR_PATH = 'mosamatic2/ui/resources/images'
MOSAMATIC2_ICONS_DIR_PATH = 'mosamatic2/ui/resources/icons'
MOSAMATIC2_APP_ICON_FILE_NAME_WIN = 'mosamatic2.ico'
MOSAMATIC2_APP_ICON_FILE_NAME_MAC = 'mosamatic2.icns'
MOSAMATIC2_BACKGROUND_IMAGE_FILE_NAME = 'body-composition.jpg'
MOSAMATIC2_BACKGROUND_IMAGE_OPACITY = 0.4

MOSAMATIC2_SERVER_PORT = 8000
MOSAMATIC2_SERVER_DEBUG = True

MOSAMATIC2_WINDOW_TITLE = 'Mosamatic'
MOSAMATIC2_WINDOW_W = 1000
MOSAMATIC2_WINDOW_H = 600
MOSAMATIC2_WINDOW_GEOMETRY_KEY = 'mainwindow/geometry'
MOSAMATIC2_WINDOW_STATE_KEY = 'mainwindow/state'

MOSAMATIC2_SPLASH_SCREEN_TITLE = MOSAMATIC2_WINDOW_TITLE
MOSAMATIC2_SPLASH_SCREEN_TITLE_STYLESHEET = 'color: rgb(64, 64, 64); font-weight: bold; font-size: 32pt;'
MOSAMATIC2_SPLASH_SCREEN_SUB_TEXT = 'This software is for research only'
MOSAMATIC2_SPLASH_SCREEN_SUB_TEXT_STYLE_SHEET = 'color: rgb(64, 64, 64); font-style: italic; font-size: 10pt;'
MOSAMATIC2_SPLASH_SCREEN_START_BUTTON_TEXT = 'Start Application'
MOSAMATIC2_SPLASH_SCREEN_QUIT_BUTTON_TEXT = 'Quit'
MOSAMATIC2_SPLASH_SCREEN_W = 600
MOSAMATIC2_SPLASH_SCREEN_H = 300